"""Autogenerated file: DO NOT EDIT!"""
import typing
from dazzler.system import Component, Aspect, UNDEFINED


class Container(Component):
    """
    Virtual div

    """
    _aspect_keys = []
    _children = []
    children = Aspect(children=True)
    id = Aspect()
    class_name = Aspect()
    style = Aspect()
    title = Aspect()
    draggable = Aspect()
    n_clicks = Aspect(default=0)

    def __init__(
            self,
            children: typing.Optional[typing.Union[str, int, float, Component,typing.List[typing.Union[str, int, float, Component]]]] = UNDEFINED,  # noqa: E501
            id: typing.Optional[str] = UNDEFINED,
            class_name: typing.Optional[str] = UNDEFINED,
            style: typing.Optional[typing.Dict] = UNDEFINED,
            title: typing.Optional[str] = UNDEFINED,
            draggable: typing.Optional[bool] = UNDEFINED,
            n_clicks: typing.Optional[typing.Union[float, int]] = 0,
            identity: str = None
    ):
        """
        :param children:
        :param id:
        :param class_name:
        :param style:
        :param title:
        :param draggable:
        :param n_clicks: (default=0)
        """
        Component.__init__(self, locals(), identity)
