.dazzler-renderer .primary {
  background-color: #5294e2; }

.dazzler-renderer .primary-light {
  background-color: #52a3f1; }

.dazzler-renderer .danger {
  background-color: #D9534F; }

.dazzler-renderer .danger-light {
  background-color: #DE6C69; }

.dazzler-renderer .warning {
  background-color: #F0AD4E; }

.dazzler-renderer .warning-light {
  background-color: #F2B968; }

.dazzler-renderer .success {
  background-color: #5CB85C; }

.dazzler-renderer .success-light {
  background-color: #74c274; }

.dazzler-renderer {
  font-family: Lato, 'Helvetica Neue', Arial, Helvetica, sans-serif; }


/*# sourceMappingURL=dazzler_commons_3261a3b3a56ab2e65425.js.map*/