"""Autogenerated file: DO NOT EDIT!"""
import typing
from dazzler.system import Component, Aspect, UNDEFINED


class Grid(Component):
    """
    Render children in a grid.

    """
    _aspect_keys = []
    _children = []
    children = Aspect(required=True, children=True)
    columns = Aspect(required=True)

    def __init__(
            self,
            children: typing.List[typing.Union[str, int, float, Component,typing.List[typing.Union[str, int, float, Component]]]],  # noqa: E501
            columns: typing.Union[float, int],
            identity: str = None
    ):
        """
        :param children: Children to render in a grid.
        :param columns: Number of columns
        """
        Component.__init__(self, locals(), identity)
