"""Autogenerated file: DO NOT EDIT!"""
import typing
from dazzler.system import Component, Aspect, UNDEFINED


class Slider(Component):
    """
    A slider with a caret contained within a min and max value.  Available CSS
    classes (dazzler-core-slider scoped): - slider-area: The outer div of the
    slider. - slider-handle: The drag handle - slider-spacer: The space that
    is before the handle.

    """
    _aspect_keys = []
    _children = []
    minimum = Aspect(required=True)
    maximum = Aspect(required=True)
    value = Aspect(default=0)
    round = Aspect()
    debounce = Aspect(default=50)
    class_name = Aspect()
    style = Aspect()

    def __init__(
            self,
            minimum: typing.Union[float, int],
            maximum: typing.Union[float, int],
            value: typing.Optional[typing.Union[float, int]] = 0,
            round: typing.Optional[typing.Any] = UNDEFINED,
            debounce: typing.Optional[typing.Union[float, int]] = 50,
            class_name: typing.Optional[str] = UNDEFINED,
            style: typing.Optional[typing.Dict] = UNDEFINED,
            identity: str = None
    ):
        """
        :param minimum: Minimum (leftmost) value of the slider.
        :param maximum: Maximum (rightmost) value of the slider.
        :param value: Current value (default=0)
        :param round: Round the value (Possible values: 'ceil', 'floor')
        :param debounce: Time in milliseconds to wait before updating the
            value. (default=50)
        :param class_name: CSS classes to use. (Scope: dazzler-core-slider)
        :param style: Style object of the root div.
        """
        Component.__init__(self, locals(), identity)
