"""Autogenerated file: DO NOT EDIT!"""
import typing
from dazzler.system import Component, Aspect, UNDEFINED


class DatePicker(Component):
    """

    """
    _aspect_keys = []
    _children = []
    date = Aspect()
    focused = Aspect()
    id = Aspect()
    placeholder = Aspect()
    aria_label = Aspect()
    disabled = Aspect()
    required = Aspect()
    read_only = Aspect()
    screen_reader_input_message = Aspect()
    show_clear_date = Aspect()
    custom_close_icon = Aspect(children=True)
    show_default_input_icon = Aspect()
    custom_input_icon = Aspect(children=True)
    input_icon_position = Aspect()
    no_border = Aspect()
    block = Aspect()
    small = Aspect()
    regular = Aspect()
    orientation = Aspect()
    anchor_direction = Aspect()
    open_direction = Aspect()
    horizontal_margin = Aspect()
    with_portal = Aspect()
    with_full_screen_portal = Aspect()
    append_to_body = Aspect()
    disable_scroll = Aspect()
    first_day_of_week = Aspect()
    number_of_months = Aspect()
    keep_open_on_date_select = Aspect()
    reopen_picker_on_clear_date = Aspect()
    hide_keyboard_shortcuts_panel = Aspect()
    day_size = Aspect()
    is_RTL = Aspect()
    transition_duration = Aspect()
    display_format = Aspect()
    month_format = Aspect()
    week_day_format = Aspect()
    phrases = Aspect()
    day_aria_label_format = Aspect()

    def __init__(
            self,
            date: typing.Optional[typing.Union[float, int]] = UNDEFINED,
            focused: typing.Optional[bool] = UNDEFINED,
            id: typing.Optional[str] = UNDEFINED,
            placeholder: typing.Optional[str] = UNDEFINED,
            aria_label: typing.Optional[str] = UNDEFINED,
            disabled: typing.Optional[bool] = UNDEFINED,
            required: typing.Optional[bool] = UNDEFINED,
            read_only: typing.Optional[bool] = UNDEFINED,
            screen_reader_input_message: typing.Optional[str] = UNDEFINED,
            show_clear_date: typing.Optional[bool] = UNDEFINED,
            custom_close_icon: typing.Optional[typing.Union[str, int, float, Component,typing.List[typing.Union[str, int, float, Component]]]] = UNDEFINED,  # noqa: E501
            show_default_input_icon: typing.Optional[bool] = UNDEFINED,
            custom_input_icon: typing.Optional[typing.Union[str, int, float, Component,typing.List[typing.Union[str, int, float, Component]]]] = UNDEFINED,  # noqa: E501
            input_icon_position: typing.Optional[typing.Any] = UNDEFINED,
            no_border: typing.Optional[bool] = UNDEFINED,
            block: typing.Optional[bool] = UNDEFINED,
            small: typing.Optional[bool] = UNDEFINED,
            regular: typing.Optional[bool] = UNDEFINED,
            orientation: typing.Optional[typing.Any] = UNDEFINED,
            anchor_direction: typing.Optional[typing.Any] = UNDEFINED,
            open_direction: typing.Optional[typing.Any] = UNDEFINED,
            horizontal_margin: typing.Optional[typing.Union[float, int]] = UNDEFINED,  # noqa: E501
            with_portal: typing.Optional[bool] = UNDEFINED,
            with_full_screen_portal: typing.Optional[bool] = UNDEFINED,
            append_to_body: typing.Optional[bool] = UNDEFINED,
            disable_scroll: typing.Optional[bool] = UNDEFINED,
            first_day_of_week: typing.Optional[typing.Any] = UNDEFINED,
            number_of_months: typing.Optional[typing.Union[float, int]] = UNDEFINED,  # noqa: E501
            keep_open_on_date_select: typing.Optional[bool] = UNDEFINED,
            reopen_picker_on_clear_date: typing.Optional[bool] = UNDEFINED,
            hide_keyboard_shortcuts_panel: typing.Optional[bool] = UNDEFINED,
            day_size: typing.Optional[typing.Union[float, int]] = UNDEFINED,
            is_RTL: typing.Optional[bool] = UNDEFINED,
            transition_duration: typing.Optional[typing.Union[float, int]] = UNDEFINED,  # noqa: E501
            display_format: typing.Optional[str] = UNDEFINED,
            month_format: typing.Optional[str] = UNDEFINED,
            week_day_format: typing.Optional[str] = UNDEFINED,
            phrases: typing.Optional[typing.Dict] = UNDEFINED,
            day_aria_label_format: typing.Optional[str] = UNDEFINED,
            identity: str = None
    ):
        """
        :param date:
        :param focused:
        :param id:
        :param placeholder:
        :param aria_label:
        :param disabled:
        :param required:
        :param read_only:
        :param screen_reader_input_message:
        :param show_clear_date:
        :param custom_close_icon:
        :param show_default_input_icon:
        :param custom_input_icon:
        :param input_icon_position: (Possible values: 'before', 'after')
        :param no_border:
        :param block:
        :param small:
        :param regular:
        :param orientation: (Possible values: 'horizontal', 'vertical')
        :param anchor_direction: (Possible values: 'left', 'right')
        :param open_direction: (Possible values: 'down', 'up')
        :param horizontal_margin:
        :param with_portal:
        :param with_full_screen_portal:
        :param append_to_body:
        :param disable_scroll:
        :param first_day_of_week: (Possible values: 0, 1, 2, 3, 4, 5, 6)
        :param number_of_months:
        :param keep_open_on_date_select:
        :param reopen_picker_on_clear_date:
        :param hide_keyboard_shortcuts_panel:
        :param day_size:
        :param is_RTL:
        :param transition_duration:
        :param display_format:
        :param month_format:
        :param week_day_format:
        :param phrases:
        :param day_aria_label_format:
        """
        Component.__init__(self, locals(), identity)
