"""
Component packages of dazzler.

- ``core``: Lightweight interactive components.
- ``extra``: Bulkier components with more functionality (markdown, syntax, etc)
- ``analytics``: Graphs and viz.
- ``media``: Video, audio,
"""
