"""Autogenerated file: DO NOT EDIT!"""
import typing
from dazzler.system import Component, Aspect, UNDEFINED


class Markdown(Component):
    """

    """
    _aspect_keys = []
    _children = []

    def __init__(
            self,
            identity: str = None
    ):
        """
        
        """
        Component.__init__(self, locals(), identity)
